#ifndef __TUPLES_H_INCLUDED
#define __TUPLES_H_INCLUDED

#include "./uint40.h"
#include "./radixsort.h"

#include <limits>

struct Triple {
  Triple() {}
  Triple(uint40 f, uint40 s, uint40 t)
    : first(f), second(s), third(t) {}
  uint40 first, second, third;
};

struct Pair {
  Pair() {}
  Pair(uint40 f, uint40 s)
    : first(f), second(s) {}
  uint40 first, second;
};

struct CmpTriple {
  inline bool operator() (const Triple &a, const Triple &b) const {
    return a.first < b.first;
  }

  Triple min_value() const {
    return Triple(std::numeric_limits<uint40>::min(), 0, 0);
  }

  Triple max_value() const {
    return Triple(std::numeric_limits<uint40>::max(), 0, 0);
  }
};

struct CmpPair {
  inline bool operator() (const Pair &a, const Pair &b) const {
    return a.first < b.first;
  }

  Pair min_value() const {
    return Pair(std::numeric_limits<uint40>::min(), 0);
  }

  Pair max_value() const {
    return Pair(std::numeric_limits<uint40>::max(), 0);
  }
};

namespace stxxl {
namespace algorithm {

template <typename Iterator>
struct Sort<Iterator, CmpTriple> {
  static inline void sort(Iterator begin, Iterator end, const CmpTriple& cmp) {
    radixsort8msb_copy2(begin, end, sizeof(uint40) - 1, cmp);
  }
};

template<typename Iterator>
struct Sort<Iterator, CmpPair> {
  static inline void sort(Iterator begin, Iterator end, const CmpPair &cmp) {
    radixsort8msb_copy2(begin, end, sizeof(uint40) - 1, cmp);
  }
};

}  // namespace algorithm
}  // namespace stxxl


#endif  // __TUPLES_H_INCLUDED
